/*
QTerm v1.10 Quake 2 Remote Console Terminal 
Copyright (c)1998 Michael Dwyer/N0ZAP (mdwyer@holly.colostate.edu)
18-May-1998

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <errno.h>
#include <string.h>

int main(int argc, char *argv[])
   {
   int                socketfd;
   int                port = htons(27910);
   char               *portstr;
   unsigned long      ipnum;
   int                len,sent;
   int                i,k;
   struct sockaddr_in hostaddress;
   unsigned char      packet[256];
   unsigned char      header[10] = {0xff, 0xff, 0xff, 0xff,
                                   'r','c','o','n',0x20,0x00};


   /*
    * Command Line Sanity Checking
    */
   if (argc < 3)
      {
      fprintf(stderr,"QTerm 1.1 by Michael Dwyer\n");
      fprintf(stderr,"Usage: %s <address>[:port] <password>\n\n", argv[0]);
      exit(1);
      }

   /*
    * Decode the address and port
    */
   portstr=strchr(argv[1],':');
   if (portstr)
      {
      *portstr++ = '\0';
      port=htons(atoi(portstr));
      fprintf(stderr,"Using port %i\n",ntohs(port));
      fprintf(stderr,"Using address %s\n", argv[1]);
      }
   else
      {
      fprintf(stderr,"Using port %i\n",ntohs(port));
      }
   ipnum=(inet_addr(argv[1]));
   if (ipnum==-1)
      {
      fprintf(stderr,"Invalid Address - Use xxx.xxx.xxx.xxx format.\n");
      fprintf(stderr,"I can't figure out domain names yet... \n");
      exit(1);
      }

   bzero((char *) &hostaddress, sizeof(hostaddress));
   hostaddress.sin_family = AF_INET;
   hostaddress.sin_addr.s_addr = ipnum; 
   hostaddress.sin_port = port;

   /*
    * Open the Socket
    */
   if ((socketfd = socket(AF_INET, SOCK_DGRAM, 0)) == -1)
      {
      perror("Couldn't open socket");
      exit(1);
      }
   
   /*
    * For these next blocks, k denotes the end of string for
    * the packet.  DO NOT RESET IT
    * j denotes the beginning of the command portion. DO NOT
    * RESET IT EITHER
    *
    * Put the header on the packet 
    */
   for (k=0;k<9;k++)
      {
      packet[k] = header[k];
      }
   /*
    * Put the password on the packet
    */
   for (i=0;i<strlen(argv[2]);i++)
      {
      packet[k] = argv[2][i];
      k++;
      }
   packet[k] = 0x20; /* Add a space */
   k++;

   while(1)
      {
      gets(&packet[k]);
      len = strlen(packet);
      len++;
      /*
       * Send the packet
       */
      sent=sendto(socketfd, packet, len ,0, 
           (struct sockaddr *)&hostaddress, sizeof(hostaddress));
      /*
       * See if it worked
       */
      if (sent!=len)
         {
         perror("Sendto failed");
         fprintf(stderr,"Tried to send %i, sent %i\n",len,sent);
         close(socketfd);
         exit(1);
         }
      }
   }
